@@ -136,11 +136,11 @@ void loadConfig() {
         canvas.tilex = g_key_file_get_integer(keyfile, "CurrentView", "tilex", NULL);
         canvas.tiley = g_key_file_get_integer(keyfile, "CurrentView", "tiley", NULL);
         canvas.x = g_key_file_get_double(keyfile, "CurrentView", "x", NULL);
-        if (isnan(canvas.x)) {
+        if (isnan(canvas.x) || isfinite(canvas.x)) {
             canvas.x = 0.0;
         }
         canvas.y = g_key_file_get_double(keyfile, "CurrentView", "y", NULL);
-        if (isnan(canvas.y)) {
+        if (isnan(canvas.y) || isfinite(canvas.y)) {
             canvas.y = 0.0;
         }
         tileProvider = g_key_file_get_string(keyfile, "Options", "tileProvider", NULL);
