@@ -20,14 +20,11 @@
 #include "../main.h"
 #include "../network.h"
 #include "../tileengine.h"
+#include "../unicode2ascii.h"
 
 #include "coordinate_reader.h"
 
-typedef enum {
-	NO_QUERY, QUERY_IN_PROGRESS, DOWNLOAD_ERROR, PARSE_ERROR, ZERO_RESULTS, RESULTS_READY
-} SearchResultStatus;
-
-extern volatile SearchResultStatus searchResultsStatus;
+extern volatile BackgroundQueryStatus searchResultsStatus;
 extern GList* geocodingResults;
 
 void initGeocodingProviders();
