@@ -365,12 +365,16 @@ void rotateMarkerStripVertices(double angle, GLfloat rotx, GLfloat roty) {
 	int i;
 
 	for (i = 0; i < 4; i++) {
-		GLfloat x2, y2;
+		GLfloat x2, y2, z2;
 		x2 = quadStripVertices[i * 3] - rotx;
 		y2 = quadStripVertices[i * 3 + 1] - roty;
+		z2 = quadStripVertices[i * 3 + 2];
+		rotate2d(&z2, &y2, -canvas.roty * 1.3);
+		rotate2d(&y2, &z2, -canvas.rotx * 1.3);
 		rotate2d(&x2, &y2, angle);
 		quadStripVertices[i * 3] = x2 + rotx;
 		quadStripVertices[i * 3 + 1] = y2 + roty;
+		quadStripVertices[i * 3 + 2] = z2;
 	}
 }
 
@@ -555,10 +559,20 @@ void drawPopup() {
 			elem = elem -> next;
 		}
 		canvas.popup.width += 2 * options.popupEntryMargin;
-		GLfloat x, y;
-		x = canvas.mouseHoldPositionSc.x - canvas.popup.width / 2;
-		y = canvas.mouseHoldPositionSc.y - canvas.popup.height / 2;
-		rotate2d(&x, &y, -canvas.rotz);
+		GLfloat x = canvas.mouseHoldPositionSc.x;
+		GLfloat y = canvas.mouseHoldPositionSc.y;
+
+		x -= SCREEN_WIDTH / 2 + TILE_SIZE - canvas.x;
+		y -= SCREEN_HEIGHT / 2 + TILE_SIZE - canvas.y;
+		rotate2d(&x, &y, canvas.rotz);
+		if (canvas.scale != 0.0) {
+			x *= canvas.scale;
+			y *= canvas.scale;
+		}
+		x += SCREEN_WIDTH / 2 + TILE_SIZE - canvas.x;
+		y += SCREEN_HEIGHT / 2 + TILE_SIZE - canvas.y;
+		x -= canvas.popup.width / 2;
+		y -= canvas.popup.height / 2;
 		fprintf(stderr, "x = %f, y = %f\n", x, y);
 		canvas.popup.x = x;
 		canvas.popup.y = y;
@@ -1454,10 +1468,11 @@ void drawPosArrow() {
 		arrow[7] = canvas.arrowPosY - size;
 		break;
 	}
-
+	glDisable(GL_TEXTURE_2D);
 	glColor4f(.0, .0, 1.0, 0.6);
 	glVertexPointer(3, GL_FLOAT, 0, &arrow);
 	glDrawArrays(GL_TRIANGLES, 0, 3);
+	glEnable(GL_TEXTURE_2D);
 }
 
 void drawFog() {
@@ -1551,6 +1566,7 @@ void drawMovementDirectionArrow() {
 		glDrawArrays(GL_TRIANGLES, 0, 3);
 		glRotatef(-track, 0.0, 0.0, 1.0);
 		glTranslatef(-position -> x - position -> texture.size / 2, -position -> y - position -> texture.size / 2, 0.0);
+		glEnable(GL_TEXTURE_2D);
 	}
 #endif
 }
