@@ -177,13 +177,17 @@ int debugPreventLoadingTiles = 0;
 
 extern SDL_sem* sem;
 
-int downloadThread(void* queue) {
+int downloadThread(void* busyPtr) {
     t_tile* tile = NULL;
     int priorityStatus = 0;
     priorityStatus = setpriority(PRIO_PROCESS, 0, 19);
+    GQueue* queue = downloadQueue;
+
+    int* busy = busyPtr;
 
 //    fprintf(stderr, "thread id: %p, setting priority: %d\n", (void*) SDL_ThreadID(), priorityStatus);
     while (!quit) {
+    	*busy = FALSE;
         tile = (t_tile*) dequeue(queue);
         while (tile == NULL) {
             //            fprintf(stderr, "queue empty, waiting, thread id = %d\n", SDL_ThreadID());
@@ -197,6 +201,7 @@ int downloadThread(void* queue) {
             SDL_Delay(20);
             tile = (t_tile*) dequeue(queue);
         }
+    	*busy = TRUE;
         int mapSize = 1 << tile -> zoom;
 
         if (tile -> tilex < 0 || tile -> tiley < 0 || tile -> tilex >= mapSize || tile -> tiley >= mapSize || debugPreventLoadingTiles) {
