@@ -99,8 +99,15 @@ double getDistance(WorldCoordinate* c1, WorldCoordinate* c2) {
 	double lat2 = degToRad(c2 -> latitude);
 	double lon1 = degToRad(c1 -> longitude);
 	double lon2 = degToRad(c2 -> longitude);
-
-	return acos(sin(lat1) * sin(lat2) + cos(lat1) * cos(lat2) * cos(lon2 - lon1)) * EARTH_RADIUS;
+	double acosarg = sin(lat1) * sin(lat2) + cos(lat1) * cos(lat2) * cos(lon2 - lon1);
+	if (acosarg > 1.0) {
+		acosarg = 1.0;
+	} else if (acosarg < -1.0) {
+		acosarg = -1.0;
+	}
+	double result = acos(acosarg) * EARTH_RADIUS;
+//	fprintf(stderr, "distance (%.20g, %.20g) - (%.20g, %.20g) = %.20g\n", c1 -> latitude, c1 -> longitude, c2 -> latitude, c2 -> longitude, result);
+	return result;
 }
 
 void toTileCoordinate(WorldCoordinate* wc, TileCoordinate* result, int zoom) {
@@ -1200,15 +1207,10 @@ void updateClickedPopupEntry() {
 		canvas.popup.clickedPopupEntry = NULL;
 		return;
 	}
-	int x, y;
+	GLfloat x, y;
+	x = mouse.x + (TILE_SIZE - canvas.x) - canvas.popup.x;
+	y = mouse.y + (TILE_SIZE - canvas.y) - canvas.popup.y;
 
-	TileCoordinate tc;
-	Point clickedPoint;
-	setClickedTileCoordinate(&tc);
-	toScreenCoordinate(&tc, &clickedPoint);
-
-	x = clickedPoint.x - canvas.popup.x;
-	y = clickedPoint.y - canvas.popup.y;
 	if (x < 0 || y < 0 || x > canvas.popup.width || y >= canvas.popup.height) {
 		canvas.popup.clickedPopupEntry = NULL;
 		return;
