@@ -2185,6 +2185,10 @@ void addIfNotZero(char** output, char c) {
 }
 
 void ascifyAndStripTags(char* input, char* output) {
+	if (input == NULL) {
+		output = "";
+		return;
+	}
 	char* inputPtr = input;
 	char* outputPtr = output;
 	unsigned c;
